#!/bin/gawk -f
#
# fraglets0.18/bin/crossm.awk
#
# crossover by homologous recombination: swaps modules of same name;
# modules are indicated in the genotype by a marker of the form
# /m modulename/
#
# Copyright (C) 2005 <christian.tschudin@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# History:
# 20050221 bug fix by lidia.yamamoto@unibas.ch
# 20050210 created by christian.tschudin@unibas.ch (fraglets v0.17)


BEGIN {
	srand();
}

	{
		s = $0;
		offs = 0;
		pos[0,"start"] = 1;
		pos[0,"length"] = 0;
		for (i = 1; match(s, "/m[ \t]+(.[^/]*)", p); i++) {
			pos[i-1,"length"] += RSTART;
			marker[p[1]] = i;
			names[i] = p[1];
			pos[i,"start"] = offs + RSTART + 1;
			pos[i,"length"] = RLENGTH -1;
			s = substr(s, RSTART+RLENGTH);
			offs += RSTART + RLENGTH - 1;
		}
		pos[i-1,"length"] += length(s);

		if ( !a ) {
			a = $0;
			for (j in marker)
				markera[j] = marker[j];
			for (j in pos)
				posa[j] = pos[j];
			delete marker;
			delete pos;
			next;
		}

		m = 1 + int((i-1) * rand() );
		#print "picking marker " m " which is " names[m];

		# crossover points:
		starta = posa[markera[names[m]],"start"];
		if (starta == "") {
		  #if marker doesn't exist in other genotype, no crossover
		  print a;
		  print $0;
		  exit;
		}
		start0 = pos[marker[names[m]],"start"];
		lena = posa[markera[names[m]],"length"];
		len0 = pos[marker[names[m]],"length"];

		# crossover result:
		print substr(a, posa[0,"start"], starta-1) \
		  substr($0, start0, len0) substr(a, starta + lena);
		print substr($0, pos[0,"start"], start0-1) \
		  substr(a, starta, lena) substr($0, start0 + len0);
		exit;
	}
