#!/bin/bash
#
# log2graphpl.sh: reaction graph generator
#
# Copyright (C) 2007 Lidia Yamamoto and Christian Tschudin
# <lidia.yamamoto@unibas.ch>, <christian.tschudin@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# Generates a reaction graph containing all species and reactions that
# occurred during the execution of a fraglet program; the output graph
# is in pdf format, and is gerated using the "neato" tool from the
# Graphviz package (http://graphviz.org/)
#
# Usage: log2graphpl.sh logfile pdffile
# * logfile: input file (output of the fraglet interpreter generated
#   with "-d 3 -e 3" options).
# * pdffile: output file in pdf format
#

if [ $# -lt 2 ]; then
    echo "usage: log2graphpl.sh logfile pdffile"
    exit -1
fi

LOG=$1
PDF=$2
DOT=`basename $PDF .pdf`.dot
PSF=`basename $PDF .pdf`.ps
LOG2GRPL=`dirname $0`/log2graph.pl

$LOG2GRPL $LOG < $LOG > $DOT
neato -Tps2 -o $PSF -Gepsilon=0.001 -Gorientation=landscape $DOT
#ps2pdf $PSF $PDF
ps2pdf -sPAPERSIZE=a1 $PSF $PDF

#rm -f $DOT
#rm -f $PSF

