#!/bin/bash
#
# adapt.sh
#
# Copyright (C) 2005 <lidia.yamamoto@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#

# same as evoloss (single allele individuals) but upgraded to uniform
# population of 8cdp + 8rdp

BIN=$HOME/unibas/src/frag-0.18/bin
SORTSG=$BIN/sortsg.awk
RENUM=$BIN/srenumber.pl
CROSSM=$BIN/crossm.awk
RMLINE=$BIN/rmline.pl
EE="$BIN/sys-send-deliver -lim 1000"
MUTATE="$BIN/mutate.pl"

TOPO=$PWD/population/topo.fra
MUTBASE=$PWD/population/mutbase.cfg
ENDMARKER=rdp_input

# 16 individuals
NGEN=50
BESTSIZE=6
CROSSIZE=14
NCROSS=4
MUTSIZE=2

#trap SIGHUP exit
#trap SIGINT exit
#trap SIGTERM exit


function nlines () {
    # nlines $fname
    fname=$1
    # get rid of verbose output of some versions of wc
    cat $fname | wc -l | perl -pe 's/[ \t]*([0-9]+).*/$1/'
}


function crossover () {
    cp $CG.crossin $CG.xx
    ncr=$NCROSS
    ncand=`nlines $CG.xx`
    while [ $ncr -gt 0 ]; do
	seln=$[$RANDOM % $ncand + 1]
	$RMLINE $seln < $CG.xx > $CG.x 2> $CG.cin
	ncand=$[$ncand - 1]

	seln=$[$RANDOM % $ncand + 1]
	$RMLINE $seln < $CG.x > $CG.xx 2>> $CG.cin
	$CROSSM < $CG.cin >> $CG.cross
	ncand=$[$ncand - 1]
	ncr=$[$ncr - 1]
    done
    rm -f $CG.xx $CG.x $CG.cin
}


function mutate () {
    # mutatemult $nmut $mfin $mfout
    # echo "mutatemult $1 $2 $3"
    nmut=$1
    mfin=$2
    mfout=$3
    tmpf="mutmult.tmp"
    # mutate $nmut random individuals in $mfin, output goes to $mfout
    nindv=`nlines $mfin`
    cp $mfin $tmpf
    while [ $nmut -gt 0 ]; do
	nindv=`nlines $tmpf`
	if [ $nindv -le 0 ]; then
	    echo "missing indvs: nmut=$nmut, empty file=$tmpf" 1>&2
	    exit
	fi
	seln=$[$RANDOM % $nindv + 1]
	$RMLINE $seln < $tmpf > ${tmpf}x 2> ${tmpf}i
	cat ${tmpf}i $MUTBASE | $MUTATE $ENDMARKER >> $mfout
	nmut=$[$nmut - 1]
	mv ${tmpf}x $tmpf
    done
    rm -f ${tmpf}*
}


function evolve () {
    N=10000
    LAST=$[$N + $NGEN]
    #echo "N=$N LAST=$LAST NGEN=$NGEN"
    
    while [ $N -lt $LAST ]; do
	CG=${N##1}
	N=$[$N + 1]
	NG=${N##1}

	if [ -s $NG.gen ]; then
	    echo "skipping lr=$lr cg=$CG ng=$NG"
	    continue
	fi

	echo "=============================="
	echo "working on lr=$lr cg=$CG ng=$NG"

	if [ -s $CG.score ]; then
	    ndone=`nlines $CG.score`
	    npop=`nlines $CG.gen`
	    ntodo=$[$npop - $ndone]
	    if [ $ntodo -gt 0 ]; then
		tail -$ntodo $CG.gen | cat $TOPO - | $EE | grep '^s' >> $CG.score
	    fi
	    $RENUM < $CG.score > $CG.snew
	    mv $CG.snew $CG.score
	else
	    cat $TOPO $CG.gen | $EE | grep '^s' > $CG.score
	fi
	sort --key=2nr $CG.score | cat $CG.gen - | $SORTSG > $CG.sort

	best=`head -1 $CG.sort | perl -pe 's/s[ \t]*([0-9\-]+).*/$1/'`
	worst=`tail -2 $CG.sort | head -1 | perl -pe 's/s[ \t]*([0-9\-]+).*/$1/'`
	echo "lr=$lr cg=$CG ng=$NG best=$best worst=$worst"
	grep '^g' $CG.sort | head -$BESTSIZE > $CG.best
	grep '^s' $CG.sort | head -$BESTSIZE > $NG.score

	grep '^g' $CG.sort | head -$CROSSIZE > $CG.crossin
	crossover
	mutate 2 $CG.crossin $CG.mut
	cat $CG.best $CG.cross $CG.mut > $NG.gen
    done
}


for lr in 00 25 ; do
    mkdir lr$lr
    cp population/pop$lr.gen lr$lr/0000.gen
    cd lr$lr
    evolve
    cd ..
done
