/* fraglets0.29/src/frag.h

  header file for fraglet interpreter core, frag.c

  Copyright (C) 2003-2007 Christian Tschudin <christian.tschudin@unibas.ch>

  This file is distributed under the terms of the GNU general Public
  License (GPL), see the file COPYING

  History:

  20070110  lidia: added node creation ("newnode")
  20061213  christian.tschudin@unibas.ch: added number support
  20050526  Jasenko Zivanov (Unibas): added socket support
  20050204  lidia.yamamoto@unibas.ch: added SYM_DEL for 'nul' op as in
            ains2003 (fraglets v0.17)
  20040704  updated by christian.tschudin@unibas.ch (fraglets v0.16)
  2003????  created by christian.tschudin@unibas.ch

*/

#ifndef FRAG_EE_H
#define FRAG_EE_H


#include <ctype.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include <time.h>
#include <unistd.h>


#define VERSION "frag v0.29 (Fraglet interpreter)\n# (c) 2003-2007 <christian.tschudin@unibas.ch>\n"

#define SYM_NUMBEROFFS (1<<10)
#define MAXSYMBOLS   (SYM_NUMBEROFFS-1)
#define MAXNODES     (1<<5)
#define MAXSEGMENTS  (1<<5)
#define MAXHINTS     50

enum { SYM_NOP = 1,
       SYM_COPY,
       SYM_DELAY,
       SYM_DUP,
       SYM_EMPTY,
       SYM_EXCH,
       SYM_LT,
       SYM_LENGTH,
       SYM_DIFF,
       SYM_SUM,
       SYM_NEWNAME,
       SYM_NEWNODE,
       SYM_MUTATE,
       SYM_NUL,
       SYM_POP,
       SYM_SEND,
       SYM_SPLIT,
       SYM_WAIT,
       SYM_TRIGGER,
       // ---------
       SYM_MATCH,
       SYM_MATCHP,
       SYM_STAR,
       SYM_NUMBER,
       SYM_EOL
};
#define SYM_LAST_TRANS SYM_TRIGGER

#define WAIT_DURATION 10
//#define WAIT_DURATION 40

typedef unsigned short symbol;

struct fragbuf {
	unsigned short refcnt;
	unsigned short len;         // in symbols
	symbol data[0];
};

struct frag {
	struct fragbuf *buf;
	struct frag *next;
	symbol nd;
	short mult;
	short offs, len; // in symbols
	char *cp;
	int expires;
};

struct m_rec {
	struct frag *first;
	struct frag *last;
	int active_cnt;
	int r_active_cnt;
};


struct o_rec {
	struct frag *first;
	struct frag *last;
	int active_cnt;
	int dummy;
};

struct t_queue {
	struct frag *first;
	struct frag *last;
	int active_cnt;
	int dummy;
};

typedef enum NodeLocation
{ Local, Remote } NodeLocation;

struct UDPNode;

struct node {
	struct node *next;
	symbol name;
	symbol segments[MAXSEGMENTS];

        // newtypeTBD attach[MAXSEGMENTS];  "prot ip port"

	struct m_rec mtab[MAXSYMBOLS];
	struct o_rec otab[MAXSYMBOLS];
	struct t_queue tq;
	struct t_queue wait;
	int r_active, t_active;
	int total_active;
	int opcnt;
	int symbalance;
	struct UDPNode* udp_wrapper;
	NodeLocation location;
	/*int socketFD;
	char ipAddress[16];*/
//	int hintcnt;
//	symbol hints[MAXHINTS];
	char possibleHint;
};

struct name {
	struct name *next;
	char *cp;
	symbol sym;
};


extern void appl_init();
extern void appl_fini();
extern  int appl_clockEvent();
extern void appl_trigger(struct frag *f);
extern void appl_scoreReset();
extern  int appl_finalScore();

#endif //FRAG_EE_H
